/*

  Program Harmonic_oscillator:
  solve a second-order linear differential equation
  for the harmonic oscillator:

  d^2 x(t) / dt^2 = - w^2 x

  with a Forward Euler, symplectic and second order
  Runge-Kutta schemes, respectively.

 */

#include <iostream>
#include <cmath>
#include <fstream>
using namespace std;

main()
{

  cout << "Solution of the equation: d^2 x/dt^2 = -w^2*x" << endl;
  cout << "with Forward-Euler, symplectic and RK2 schemes" << endl;

  cout << "Insert dt, Tend, Tprint: ";
  double dt, Tend, Tprint;
  cin >> dt >> Tend >> Tprint;

  long Nend   = long( Tend / dt );
  long Nprint = long( Tprint / dt );

  cout << "Insert parameter w: ";
  double w;
  cin >> w;
  double w2 = w * w;

  cout << "Insert initial conditions x0, v0: ";
  double x0, v0;
  cin >> x0 >> v0;

  cout << "Output file name: ";
  char nfile[256];
  cin >> nfile;

  cout << "Inser the scheme to use:" << endl;
  cout << "  1 - Forward Euler" << endl;
  cout << "  2 - Symplectic" << endl;
  cout << "  3 - Second order Runge-Kutta" << endl;
  int scheme;
  cin >> scheme;

  // Initial variables

  double t_n, t_star, x_n, x_np1, x_star, v_n, v_np1, v_star;
  double A, phi, x_ex, err, Energy, E0;

  t_n  = 0.0;
  x_n  = x0;
  v_n  = v0;

  if ( v0 != 0.0 )
    {
      A = sqrt( pow( x0, 2 ) + pow( ( v0 / w ), 2.0 ) );
      phi = atan( x0 * w / v0 );
    }
  else
    {
      A = x0;
      phi = acos( -1.0 ) / 2.0;
    }
  x_ex = A * sin( w * t_n + phi );

  err = abs( x_n - x_ex );
  Energy = 0.5 * ( v_n * v_n + w2 * x_n * x_n );
  E0 = Energy;

  // Open the output file and writes the initial conditions

  ofstream fout;
  fout.open( nfile );
  fout.precision( 15 );
  fout << "#   t     x     v     x_ex    err    DEn" << endl;
  fout << t_n << "   " << x_n << "    " << v_n << "   " << x_ex << "   " << err 
       << "   " << Energy - E0 << endl;

  // Time loop

  for ( long n = 1; n <= Nend; n++ )
    {

      // Computes the solution at next time-step according to the scheme

      switch ( scheme )
	{
	case 1:    // Forward Euler
	  x_np1 = x_n + dt * v_n;
	  v_np1 = v_n - w2 * dt * x_n;
	  break;
	case 2:    // Symplectic
	  x_np1 = x_n + dt * v_n;
	  v_np1 = v_n - w2 * dt * x_np1;
	  break;
	case 3:    // Second order Runge-Kutta
	  x_star = x_n + 0.5 * dt * v_n;
	  v_star = v_n - w2 * 0.5 * dt * x_n;
	  t_star = t_n + 0.5 * dt;
	  x_np1 = x_n + dt * v_star;
	  v_np1 = v_n - w2 * dt * x_star;
	  break;
	}

      t_n = n * dt;
      x_n = x_np1;
      v_n = v_np1;

      // Prints the numerical and analytical solutions at each print time

      if ( ( n % Nprint ) == 0 )
	{
          x_ex = A * sin( w * t_n + phi );
	  err = abs( x_n - x_ex );
          Energy = 0.5 * ( v_n * v_n + w2 * x_n * x_n );
	  fout << t_n << "   " << x_n << "    " << v_n << "   " << x_ex << "   " << err 
               << "   " << Energy - E0 << endl;
	}

    }

  fout.close();

}
